@@ -23,4 +23,13 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-__version__ = (0, 10, 0)
+__version__ = (0, 11, 0)
+
+# Use simplejson or Python 2.6 json
+# simplejson is much faster that py26:json. so use simplejson if available
+try:
+    import simplejson
+    json = simplejson
+except ImportError: # pragma: no cover
+    import json
+json # pyflakes
