@@ -1,44 +1,30 @@
 """Manage (save/check) task dependency-on-files data."""
 
 import os
-import anydbm
+import hashlib
+import dumbdbm
+import anydbm as ddbm
 
-#FIXME move json import to __init__.py
-# Use simplejson or Python 2.6 json
-# simplejson is much faster that py26:json. so use simplejson if available
-try:
-    import simplejson
-    json = simplejson
-except ImportError: # pragma: no cover
-    import json
-
-
-############ md5
-def get_md5_py5(input_data):
-    """return md5 from string (python 2.5 and above)"""
-    return hashlib.md5(input_data).hexdigest()
-
-def get_md5_py4(input_data): # pragma: no cover
-    """return md5 from string"""
-    out = md5.new()
-    out.update(input_data)
-    return out.hexdigest()
-
-## use different md5 libraries depending on python version
-get_md5 = None # function return md5 from a string
-try:
-    import hashlib
-    get_md5 = get_md5_py5
-# support python 2.4
-except ImportError: # pragma: no cover
-    import md5
-    get_md5 = get_md5_py4
-#########################################################
+# uncomment imports below to run tests on all dbm backends...
+#import dbhash as ddbm # ok (removed from python3)
+#import dumbdbm as ddbm # test_corrupted_file fails
+#import dbm as ddbm # test_corrupted_file fails
+#import gdbm as ddbm # ok <= TODO make this the default
 
+from doit import json
 
 
 USE_FILE_TIMESTAMP = True
 
+
+def get_md5(input_data):
+    """return md5 from string or unicode"""
+    if isinstance(input_data, unicode):
+        byte_data = input_data.encode("utf-8")
+    else:
+        byte_data = input_data
+    return hashlib.md5(byte_data).hexdigest()
+
 def md5sum(path):
     """Calculate the md5 sum from file content.
 
@@ -153,11 +139,29 @@ class DbmDB(object):
     @ivar _db: (dict) items with python-dict as value
     @ivar dirty: (set) id of modified tasks
     """
+    DBM_CONTENT_ERROR_MSG = 'db type could not be determined'
 
     def __init__(self, name):
         """Open/create a DB file"""
         self.name = name
-        self._dbm = anydbm.open(self.name, 'c')
+        try:
+            self._dbm = ddbm.open(self.name, 'c')
+        except ddbm.error, exception:
+            message = str(exception)
+            if message == self.DBM_CONTENT_ERROR_MSG:
+                # When a corrupted/old format database is found
+                # suggest the user to just remove the file
+                new_message = (
+                    'Dependencies file in %(filename)s seems to use '
+                    'an old format or is corrupted.\n'
+                    'To fix the issue you can just remove the database file(s) '
+                    'and a new one will be generated.'
+                    % {'filename': repr(self.name)})
+                raise exception.__class__, new_message
+            else:
+                # Re-raise any other exceptions
+                raise
+
         self._db = {}
         self.dirty = set()
 
@@ -174,6 +178,11 @@ class DbmDB(object):
         self._db[task_id][dependency] = value
         self.dirty.add(task_id)
 
+    def _in_dbm(self, key):
+        """python3: when for get/set key is convert to bytes but not for 'in'"""
+        # should be just
+        # return key in self._dbm
+        return key.encode('utf-8') in self._dbm
 
     def get(self, task_id, dependency):
         """Get value stored in the DB.
@@ -184,22 +193,22 @@ class DbmDB(object):
         if task_id in self._db:
             return self._db[task_id].get(dependency, None)
 
-        if task_id not in self._db and task_id in self._dbm:
-            self._db[task_id] = json.loads(self._dbm[task_id])
+        if task_id not in self._db and self._in_dbm(task_id):
+            self._db[task_id] = json.loads(self._dbm[task_id].decode('utf-8'))
         if task_id in self._db:
             return self._db[task_id].get(dependency, None)
 
 
     def in_(self, task_id):
         """@return bool if task_id is in DB"""
-        return task_id in self._dbm or task_id in self.dirty
+        return self._in_dbm(task_id) or task_id in self.dirty
 
 
     def remove(self, task_id):
         """remove saved dependecies from DB for taskId"""
         if task_id in self._db:
             del self._db[task_id]
-        if task_id in self._dbm:
+        if self._in_dbm(task_id):
             del self._dbm[task_id]
         if task_id in self.dirty:
             self.dirty.remove(task_id)
@@ -207,7 +216,11 @@ class DbmDB(object):
     def remove_all(self):
         """remove saved dependecies from DB for all tasks"""
         self._db = {}
-        self._dbm = anydbm.open(self.name, 'n')
+        # dumb dbm always opens file in update mode
+        if isinstance(self._dbm, dumbdbm._Database): # pragma: no cover
+            self._dbm._index = {}
+            self._dbm.close()
+        self._dbm = ddbm.open(self.name, 'n')
         self.dirty = set()
 
 
@@ -215,7 +228,7 @@ class DbmDB(object):
 class DependencyBase(object):
     """Manage tasks dependencies (abstract class)
 
-    Each dependency is a saved in "db". the "db" is a text file using json
+    Each dependency is a saved in "db". the "db" can have json or dbm
     format where there is a dictionary for every task. each task has a
     dictionary where key is a dependency (abs file path), and the value is the
     dependency signature.
@@ -235,6 +248,7 @@ class DependencyBase(object):
         self.remove = self.backend.remove
         self.remove_all = self.backend.remove_all
         self._in = self.backend.in_
+        self.name = self.backend.name
 
     def close(self):
         """Write DB in file"""
@@ -285,7 +299,7 @@ class DependencyBase(object):
         """get saved value from task
         @param name (str): taskid.argument-name
         """
-        parts = name.split('.')
+        parts = name.rsplit('.', 1)
         assert len(parts) == 2
         taskid, arg_name = parts
         if not self._in(taskid):
@@ -305,6 +319,7 @@ class DependencyBase(object):
         self._set(task.name, 'ignore:', '1')
 
 
+    # TODO add option to log this
     def get_status(self, task):
         """Check if task is up to date. set task.dep_changed
 
@@ -379,5 +394,11 @@ class DbmDependency(DependencyBase):
     def __init__(self, name):
         DependencyBase.__init__(self, DbmDB(name))
 
-# default "Dependency" implementation to be used
-Dependency = DbmDependency
+import platform
+python_version = platform.python_version().split('.')
+if python_version[0] == '2' and python_version[1] == '5': # pragma: no cover
+    # use json by default on python2.5 because gdbm in python2.5 is broken
+    Dependency = JsonDependency
+else:
+    Dependency = DbmDependency
+
