@@ -5,7 +5,7 @@ import sys
 import traceback
 
 import doit
-from doit.exceptions import InvalidDodoFile, InvalidCommand
+from doit.exceptions import InvalidDodoFile, InvalidCommand, InvalidTask
 from doit import loader
 from doit import cmdparse
 from doit.cmds import doit_run, doit_clean, doit_list, doit_forget, doit_ignore
@@ -324,7 +324,8 @@ def cmd_auto(params, args):
                                   params['sub'].keys())
     params.update_defaults(dodo_tasks['config'])
     filter_tasks = args or dodo_tasks['config'].get('default_tasks')
-    return doit_auto(params['dep_file'], dodo_tasks['task_list'], filter_tasks)
+    return doit_auto(params['dep_file'], dodo_tasks['task_list'], filter_tasks,
+                     params['verbosity'])
 
 
 ##########################################################
@@ -467,7 +468,7 @@ def cmd_main(cmd_args):
                                         cmd_ignore, ignore_doc)
 
     # auto command
-    auto_options = (opt_dodo, opt_cwd, opt_depfile,)
+    auto_options = (opt_dodo, opt_cwd, opt_depfile, opt_verbosity)
     sub_cmd['auto'] = cmdparse.Command('auto', auto_options,
                                         cmd_auto, auto_doc)
 
@@ -476,15 +477,9 @@ def cmd_main(cmd_args):
     try:
         return sub_cmd['run'](cmd_args, sub=sub_cmd)
 
-    # in python 2.4 SystemExit and KeyboardInterrupt subclass
-    # from Exception.
-    except (SystemExit, KeyboardInterrupt):
-        raise
-
     # dont show traceback for user errors.
-    # TODO check no InvalidTask exception endup here.
     except (cmdparse.CmdParseError, InvalidDodoFile,
-            InvalidCommand), err:
+            InvalidCommand, InvalidTask), err:
         sys.stderr.write("ERROR: %s\n" % str(err))
         return 1
 
