@@ -1,7 +1,7 @@
 import sys
 import inspect
 
-import py.test
+import pytest
 from mock import Mock
 
 from doit.exceptions import InvalidCommand
@@ -191,17 +191,19 @@ class TestInterface(object):
         monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
         mock_auto = Mock()
         monkeypatch.setattr(doit_cmd, "doit_auto", mock_auto)
-        doit_cmd.cmd_main(["auto", "b"])
+        doit_cmd.cmd_main(["auto", "-v", "2", "b"])
 
         expected = [('dependency_file', '.doit.db'),
                     ('task_list', mock_get_tasks()['task_list']),
                     ('filter_tasks', ['b']),
+                    ('verbosity', 2),
+                    ('reporter', 'executed-only'),
                     ('loop_callback', None),
                     ]
 
         assert len(expected) == len(argspec[0])
-        #                                                    + loop_callback
-        assert len(expected) == (len(mock_auto.call_args[0]) + 1)
+        #                                    + reporter + loop_callback
+        assert len(expected) == (len(mock_auto.call_args[0]) + 2)
         for exp,got in zip(expected, zip(argspec[0], mock_auto.call_args[0])):
             assert exp == got
 
@@ -213,7 +215,7 @@ class TestErrors(object):
             raise KeyboardInterrupt()
         mock_cmd = Mock(side_effect=my_raise)
         monkeypatch.setattr(doit_cmd, "doit_run", mock_cmd)
-        py.test.raises(KeyboardInterrupt, doit_cmd.cmd_main, [])
+        pytest.raises(KeyboardInterrupt, doit_cmd.cmd_main, [])
 
     def test_user_error(self, capsys, monkeypatch):
         monkeypatch.setattr(loader, "get_tasks", mock_get_tasks)
