@@ -1,8 +1,10 @@
+# coding=UTF-8
+
 import os, shutil
 import tempfile
 from StringIO import StringIO
 
-import py.test
+import pytest
 
 from doit.exceptions import TaskError
 from doit.exceptions import CatchedException
@@ -25,11 +27,11 @@ class TestTaskCheckInput(object):
         task.Task.check_attr_input('xxx', 'attr', None, ([list], [None]))
 
     def testFailType(self):
-        py.test.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
+        pytest.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
                       'attr', int, ([list], [False]))
 
     def testFailValue(self):
-        py.test.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
+        pytest.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
                       'attr', True, ([list], [False]))
 
 
@@ -48,7 +50,7 @@ class TestTaskInit(object):
     # give proper error message when anything else is used.
     def test_dependencyNotSequence(self):
         filePath = "data/dependency1"
-        py.test.raises(task.InvalidTask, task.Task,
+        pytest.raises(task.InvalidTask, task.Task,
                       "Task X",["taskcmd"], file_dep=filePath)
 
     def test_options(self):
@@ -83,13 +85,6 @@ class TestTaskUpToDate(object):
         assert not t.run_once
         assert t.uptodate == [None]
 
-    # deprecate bool and none values for file_dep. to be removed on 0.11
-    def test_deprecation(self):
-        t = task.Task("Task x", ["taskcmd"], file_dep=[True, False, None])
-        assert 0 == len(t.file_dep)
-        assert True == t.run_once
-        assert [False, None] == t.uptodate
-
 class TestTaskExpandFileDep(object):
 
     def test_dependencyStringIsFile(self):
@@ -97,13 +92,18 @@ class TestTaskExpandFileDep(object):
         assert set(["123","456"]) == my_task.file_dep
 
     def test_runOnce_or_fileDependency(self):
-        py.test.raises(task.InvalidTask, task.Task, "Task X",
+        pytest.raises(task.InvalidTask, task.Task, "Task X",
                        ["taskcmd"], file_dep=["whatever"], run_once=True)
 
     def test_file_dep_must_be_string(self):
-        py.test.raises(task.InvalidTask, task.Task, "Task X", ["taskcmd"],
+        pytest.raises(task.InvalidTask, task.Task, "Task X", ["taskcmd"],
                        file_dep=[['aaaa']])
 
+    def test_file_dep_unicode(self):
+        unicode_name = u"中文"
+        my_task = task.Task("Task X", ["taskcmd"], file_dep=[unicode_name])
+        assert unicode_name in my_task.file_dep
+
 
 class TestTaskDeps(object):
 
@@ -148,9 +148,14 @@ class TestTask_Getargs(object):
 
     def test_invalid_desc(self):
         getargs = {'x' : 't1'}
-        assert py.test.raises(task.InvalidTask, task.Task,
+        assert pytest.raises(task.InvalidTask, task.Task,
                               't3', None, getargs=getargs)
 
+    def test_many_dots(self):
+        getargs = {'x': 't2:file.ext.x'}
+        t = task.Task('t1', None, getargs=getargs)
+        assert 't2:file.ext' in t.task_dep
+
 
 class TestTaskTitle(object):
 
@@ -221,7 +226,8 @@ class TestTaskActions(object):
     # python and commands mixed on same task
     def test_mixed(self):
         def my_print(msg):
-            print msg,
+            import sys # python3 2to3 cant handle print with a trailing comma
+            sys.stdout.write(msg)
         t = task.Task("taskX",["%s hi_stdout hi2" % PROGRAM,
                                (my_print,['_PY_']),
                                "%s hi_list hi6" % PROGRAM])
@@ -258,7 +264,7 @@ class TestTaskClean(object):
             tmpdir['files'] = files
             # create empty files
             for filename in tmpdir['files']:
-                file(filename, 'a').close()
+                open(filename, 'a').close()
             return tmpdir
         def remove_tmpdir(tmpdir):
             if os.path.exists(tmpdir['dir']):
@@ -317,7 +323,7 @@ class TestTaskClean(object):
         # a clean action can be anything, it can even not clean anything!
         c_path = tmpdir['files'][0]
         def say_hello():
-            fh = file(c_path, 'a')
+            fh = open(c_path, 'a')
             fh.write("hello!!!")
             fh.close()
         t = task.Task("xxx",None,targets=tmpdir['files'], clean=[(say_hello,)])
@@ -326,7 +332,7 @@ class TestTaskClean(object):
         t.clean(StringIO(), False)
         for filename in tmpdir['files']:
             assert os.path.exists(filename)
-        fh = file(c_path, 'r')
+        fh = open(c_path, 'r')
         got = fh.read()
         fh.close()
         assert "hello!!!" == got
@@ -391,8 +397,8 @@ class TestDictToTask(object):
 
     def testDictFieldTypo(self):
         dict_ = {'name':'z','actions':['xpto 14'],'typo_here':['xxx']}
-        py.test.raises(action.InvalidTask, task.dict_to_task, dict_)
+        pytest.raises(action.InvalidTask, task.dict_to_task, dict_)
 
     def testDictMissingFieldAction(self):
-        py.test.raises(action.InvalidTask, task.dict_to_task, {'name':'xpto 14'})
+        pytest.raises(action.InvalidTask, task.dict_to_task, {'name':'xpto 14'})
 
